<?php

namespace App\Imports;

use Illuminate\Support\Collection;

use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithMapping;
use Maatwebsite\Excel\Concerns\WithStartRow;
use Maatwebsite\Excel\Concerns\WithMultipleSheets;

/**
 * 導入
 *
 * @example https://docs.laravel-excel.com/3.1/imports/ 文檔
 * @example https://phpspreadsheet.readthedocs.io/en/latest/ PhpSpreadsheet 文檔
 */
class MemberImport implements ToCollection, WithStartRow, WithMapping, WithMultipleSheets
{
    /**
     * $config.
     *
     * @var array
     */
	protected $config;

	public function __construct()
	{
		$this->config = config('member');
	}

    /**
     * 格式化 - 列
     */
    public function map($row): array
    {
    	$keys = ['account', 'password', 'name', 'sex', 'birthday', 'tel', 'country', 'town', 'zip', 'address'];

    	return combine_array_by_key($keys, $row);
    }

    /**
     * 資料集合
     */
    public function collection(Collection $rows)
    {
        $rule = [
            'account' => [
            	'required',
				'email',
				\Rule::unique($this->config['table'])->where(function($query) {
					return $query->where([['del', 0]]);
				}),
            ],
			'password' => 'required',
			'name'     => 'required',
			'sex'      => [
				'required',
				\Rule::in([1, 2]),
			],
        ];
        $rule_message = [
			'account.required'  => '帳號 尚未填寫',
			'account.email'     => '帳號 不符合Email格式',
			'account.unique'    => '帳號已被使用！',
			'password.required' => '密碼 尚未填寫',
			'name.required'     => '姓名 尚未填寫',
			'sex.required'      => '性別 尚未填寫',
			'sex.in'            => '性別 設定錯誤',

        ];
    	$error_message = [];

    	// ------------------------------------ 資料檢查 ------------------------------------
        foreach ($rows as $row_num => $row)
        {
        	# 每列驗證檢查
        	$validator = \Validator::make($row->toArray(), $rule, $rule_message);

        	if ($validator->fails()) {
        		foreach ($validator->errors()->all() as $value) {
        			$error_message[] = '第' . ($row_num + 2) . '列 ' . $value;
        		}
        	}
        }

            # 重複檢查
        $total = $rows->count();

        if($rows->uniqueStrict('account')->count() != $total) {
            $error_message[] = '帳號 有重複值';
        }

		// ------------------------------------ 資料設定和寫入 ------------------------------------

		if(count($error_message) > 0) {
			return back()->withErrors($error_message)->send();
		} else {
			$insert_total = 0;

			foreach ($rows as $row) {
				$insert_data = $row->filter(function($item, $key) {
										return !is_null($item);
									})
									->except([''])
									->toArray();
				// 密碼加密
				$insert_data['password'] = \Hash::make($insert_data['password']);;
				// 黑名單
				$insert_data['block'] = 0;

				if($id = \DB::table($this->config['table'])->insertGetId($insert_data)) {
					$insert_total++;
				}
			}

			return back()->with('message', '匯入成功,共建立 ' . $insert_total . ' 筆資料')->send();
		}
    }

    /**
     * 設定開始的列
     */
    public function startRow(): int
    {
        return 2;
    }

    /**
     * 工作表 選擇
     *
     * 目前只處理第一張工作表
     */
    public function sheets(): array
    {
        return [
            0 => $this,
        ];
    }
}